@@ -2,6 +2,10 @@
 """
 # Standard library imports ...
 import doctest
+try:
+    import importlib.resources as ir
+except ImportError:
+    import importlib_resources as ir
 from io import BytesIO
 import os
 import re
@@ -12,15 +16,13 @@ import tempfile
 from uuid import UUID
 import unittest
 import warnings
-try:
-    # Third party library import, favored over standard library.
-    import lxml.etree as ET
-except ImportError:
-    import xml.etree.ElementTree as ET
 
 # Third party library imports ...
 import numpy as np
-import pkg_resources as pkg
+try:
+    from lxml import etree as ET
+except ImportError:
+    import xml.etree.ElementTree as ET
 
 # Local imports ...
 import glymur
@@ -29,9 +31,11 @@ from glymur.jp2box import ColourSpecificationBox, ContiguousCodestreamBox
 from glymur.jp2box import FileTypeBox, ImageHeaderBox, JP2HeaderBox
 from glymur.jp2box import JPEG2000SignatureBox, BitsPerComponentBox
 from glymur.jp2box import PaletteBox, UnknownBox, CaptureResolutionBox
+from glymur.jp2box import InvalidJp2kError
 from glymur.core import COLOR, OPACITY, SRGB, GREYSCALE
 from glymur.core import RED, GREEN, BLUE, GREY, WHOLE_IMAGE
-from .fixtures import WINDOWS_TMP_FILE_MSG, MetadataBase
+from . import fixtures, data
+from .fixtures import MetadataBase
 
 
 def docTearDown(doctest_obj):
@@ -48,11 +52,8 @@ def load_tests(loader, tests, ignore):
     return tests
 
 
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestDataEntryURL(unittest.TestCase):
+class TestDataEntryURL(fixtures.TestCommon):
     """Test suite for DataEntryURL boxes."""
-    def setUp(self):
-        self.jp2file = glymur.data.nemo()
 
     @unittest.skipIf(re.match("1.5|2",
                               glymur.version.openjpeg_version) is None,
@@ -64,14 +65,14 @@ class TestDataEntryURL(unittest.TestCase):
         red = data[:, :, 0]
 
         # Write it back out as a raw codestream.
-        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile1:
-            j2k = glymur.Jp2k(tfile1.name, data=red)
+        with open(self.temp_j2k_filename, mode="wb") as tfile:
+            j2k = glymur.Jp2k(tfile.name, data=red)
 
-            # Ok, now rewrap it as JP2.  The colorspace should be GREYSCALE.
-            with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile2:
-                jp2 = j2k.wrap(tfile2.name)
-                self.assertEqual(jp2.box[2].box[1].colorspace,
-                                 glymur.core.GREYSCALE)
+        # Ok, now rewrap it as JP2.  The colorspace should be GREYSCALE.
+        with open(self.temp_jp2_filename, mode="wb") as tfile2:
+            jp2 = j2k.wrap(tfile2.name)
+            self.assertEqual(jp2.box[2].box[1].colorspace,
+                             glymur.core.GREYSCALE)
 
     def test_basic_url(self):
         """Just your most basic URL box."""
@@ -82,7 +83,7 @@ class TestDataEntryURL(unittest.TestCase):
         deurl = glymur.jp2box.DataEntryURLBox(0, (0, 0, 0), url)
         boxes = [box for box in jp2.box if box.box_id != 'uuid']
         boxes.append(deurl)
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             jp22 = jp2.wrap(tfile.name, boxes=boxes)
 
         actdata = [box.box_id for box in jp22.box]
@@ -100,7 +101,7 @@ class TestDataEntryURL(unittest.TestCase):
         deurl = glymur.jp2box.DataEntryURLBox(0, (0, 0, 0), url)
         boxes = [box for box in jp2.box if box.box_id != 'uuid']
         boxes.append(deurl)
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             jp22 = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jp22.box[-1].length, 42)
@@ -121,8 +122,7 @@ class TestDataEntryURL(unittest.TestCase):
 @unittest.skipIf(re.match(r'''0|1|2.0.0''',
                           glymur.version.openjpeg_version) is not None,
                  "Not supported until 2.1")
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestChannelDefinition(unittest.TestCase):
+class TestChannelDefinition(fixtures.TestCommon):
     """Test suite for channel definition boxes."""
 
     @classmethod
@@ -130,31 +130,30 @@ class TestChannelDefinition(unittest.TestCase):
         """Need a one_plane plane image for greyscale testing."""
         j2k = Jp2k(glymur.data.goodstuff())
         data = j2k[:]
+
+        cls.channel_def_test_dir = tempfile.mkdtemp()
+
         # Write the first component back out to file.
-        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
-            Jp2k(tfile.name, data=data[:, :, 0])
-            cls.one_plane = tfile.name
+        cls.one_plane = os.path.join(cls.channel_def_test_dir, 'one_plane.j2k')
+        Jp2k(cls.one_plane, data=data[:, :, 0])
+
         # Write the first two components back out to file.
-        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
-            Jp2k(tfile.name, data=data[:, :, 0:1])
-            cls.two_planes = tfile.name
+        cls.two_planes = os.path.join(cls.channel_def_test_dir, 'two.j2k')
+        Jp2k(cls.two_planes, data=data[:, :, 0:1])
+
         # Write four components back out to file.
-        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
-            shape = (data.shape[0], data.shape[1], 1)
-            alpha = np.zeros((shape), dtype=data.dtype)
-            data4 = np.concatenate((data, alpha), axis=2)
-            Jp2k(tfile.name, data=data4)
-            cls.four_planes = tfile.name
+        cls.four_planes = os.path.join(cls.channel_def_test_dir, 'four.j2k')
+        shape = (data.shape[0], data.shape[1], 1)
+        alpha = np.zeros((shape), dtype=data.dtype)
+        data4 = np.concatenate((data, alpha), axis=2)
+        Jp2k(cls.four_planes, data=data4)
 
     @classmethod
     def tearDownClass(cls):
-        os.unlink(cls.one_plane)
-        os.unlink(cls.two_planes)
-        os.unlink(cls.four_planes)
+        shutil.rmtree(cls.channel_def_test_dir)
 
     def setUp(self):
-        self.jp2file = glymur.data.nemo()
-        self.j2kfile = glymur.data.goodstuff()
+        super(TestChannelDefinition, self).setUp()
 
         j2k = Jp2k(self.j2kfile)
         codestream = j2k.get_codestream()
@@ -171,9 +170,6 @@ class TestChannelDefinition(unittest.TestCase):
         self.colr_rgb = ColourSpecificationBox(colorspace=SRGB)
         self.colr_gr = ColourSpecificationBox(colorspace=GREYSCALE)
 
-    def tearDown(self):
-        pass
-
     def test_cdef_no_inputs(self):
         """channel_type and association are required inputs."""
         with self.assertRaises(TypeError):
@@ -190,7 +186,7 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_rgb, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
 
             jp2 = Jp2k(tfile.name)
@@ -213,7 +209,7 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_rgb, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
 
             jp2 = Jp2k(tfile.name)
@@ -236,7 +232,7 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_rgb, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
 
             jp2 = Jp2k(tfile.name)
@@ -257,8 +253,8 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_rgb, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_grey(self):
@@ -271,7 +267,7 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_gr, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
 
             jp2 = Jp2k(tfile.name)
@@ -292,7 +288,7 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_gr, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
 
             jp2 = Jp2k(tfile.name)
@@ -316,8 +312,8 @@ class TestChannelDefinition(unittest.TestCase):
         boxes = [self.ihdr, self.colr_gr, cdef]
         self.jp2h.box = boxes
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises((OSError, IOError)):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises((OSError, InvalidJp2kError)):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_only_one_cdef_in_jp2h(self):
@@ -334,8 +330,8 @@ class TestChannelDefinition(unittest.TestCase):
 
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
 
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_not_in_jp2h(self):
@@ -351,29 +347,25 @@ class TestChannelDefinition(unittest.TestCase):
 
         boxes = [self.jp2b, self.ftyp, self.jp2h, cdef, self.jp2c]
 
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises((IOError, OSError)):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises((InvalidJp2kError, OSError)):
                 j2k.wrap(tfile.name, boxes=boxes)
 
 
-class TestFileTypeBox(unittest.TestCase):
+class TestFileTypeBox(fixtures.TestCommon):
     """Test suite for ftyp box issues."""
 
-    def setUp(self):
-        self.jp2file = glymur.data.nemo()
-
     def test_bad_brand_on_parse(self):
         """The JP2 file file type box does not contain a valid brand.
 
         Expect a specific validation error.
         """
-        relpath = os.path.join('data', 'issue396.jp2')
-        filename = pkg.resource_filename(__name__, relpath)
-        with warnings.catch_warnings():
-            # Lots of things wrong with this file.
-            warnings.simplefilter('ignore')
-            with self.assertRaises(IOError):
-                Jp2k(filename)
+        with ir.path(data, 'issue396.jp2') as path:
+            with warnings.catch_warnings():
+                # Lots of things wrong with this file.
+                warnings.simplefilter('ignore')
+                with self.assertRaises(InvalidJp2kError):
+                    Jp2k(path)
 
     def test_brand_unknown(self):
         """A ftyp box brand must be 'jp2 ' or 'jpx '."""
@@ -381,7 +373,7 @@ class TestFileTypeBox(unittest.TestCase):
             warnings.simplefilter('ignore')
             ftyp = glymur.jp2box.FileTypeBox(brand='jp3')
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 ftyp.write(tfile)
 
     def test_cl_entry_unknown(self):
@@ -391,10 +383,9 @@ class TestFileTypeBox(unittest.TestCase):
             # Bad compatibility list item.
             ftyp = glymur.jp2box.FileTypeBox(compatibility_list=['jp3'])
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 ftyp.write(tfile)
 
-    @unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
     @unittest.skipIf(sys.hexversion < 0x03000000,
                      "assertWarns not introduced until 3.2")
     def test_cl_entry_not_utf8(self):
@@ -404,7 +395,7 @@ class TestFileTypeBox(unittest.TestCase):
 
         # Replace bytes 28-32 with bad utf-8 data
         data = data[:28] + b'\xff\xff\xff\xff' + data[32:]
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             tfile.write(data)
             tfile.flush()
 
@@ -412,11 +403,11 @@ class TestFileTypeBox(unittest.TestCase):
                 Jp2k(tfile.name)
 
 
-class TestColourSpecificationBox(unittest.TestCase):
+class TestColourSpecificationBox(fixtures.TestCommon):
     """Test suite for colr box instantiation."""
 
     def setUp(self):
-        self.j2kfile = glymur.data.goodstuff()
+        super(TestColourSpecificationBox, self).setUp()
 
         j2k = Jp2k(self.j2kfile)
         codestream = j2k.get_codestream()
@@ -437,46 +428,42 @@ class TestColourSpecificationBox(unittest.TestCase):
 
         It's enough that it doesn't error out.
         """
-        relpath = os.path.join('data', 'issue405.dat')
-        filename = pkg.resource_filename(__name__, relpath)
-        with open(filename, 'rb') as f:
-            f.seek(8)
-            with warnings.catch_warnings():
-                # Lots of things wrong with this file.
-                warnings.simplefilter('ignore')
-                box = ColourSpecificationBox.parse(f, length=80, offset=0)
+        with ir.path(data, 'issue405.dat') as path:
+            with path.open('rb') as f:
+                f.seek(8)
+                with warnings.catch_warnings():
+                    # Lots of things wrong with this file.
+                    warnings.simplefilter('ignore')
+                    box = ColourSpecificationBox.parse(f, length=80, offset=0)
         str(box)
 
-    @unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
     def test_colr_with_out_enum_cspace(self):
         """must supply an enumerated colorspace when writing"""
         j2k = Jp2k(self.j2kfile)
 
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
         boxes[2].box = [self.ihdr, ColourSpecificationBox(colorspace=None)]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
-    @unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
     def test_missing_colr_box(self):
         """jp2h must have a colr box"""
         j2k = Jp2k(self.j2kfile)
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
         boxes[2].box = [self.ihdr]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
-    @unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
     def test_bad_approx_jp2_field(self):
         """JP2 has requirements for approx field"""
         j2k = Jp2k(self.j2kfile)
         boxes = [self.jp2b, self.ftyp, self.jp2h, self.jp2c]
         colr = ColourSpecificationBox(colorspace=SRGB, approximation=1)
         boxes[2].box = [self.ihdr, colr]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_default_colr(self):
@@ -491,10 +478,7 @@ class TestColourSpecificationBox(unittest.TestCase):
     def test_icc_profile_data(self):
         """basic colr box with ICC profile"""
 
-        relpath = os.path.join('data', 'sgray.icc')
-        iccfile = pkg.resource_filename(__name__, relpath)
-        with open(iccfile, mode='rb') as f:
-            raw_icc_profile = f.read()
+        raw_icc_profile = ir.read_binary(data, 'sgray.icc')
 
         colr = ColourSpecificationBox(icc_profile_data=raw_icc_profile)
         self.assertEqual(colr.method, glymur.core.ENUMERATED_COLORSPACE)
@@ -508,39 +492,38 @@ class TestColourSpecificationBox(unittest.TestCase):
         self.assertEqual(len(colr.icc_profile_data), 416)
 
     def test_colr_with_bad_color(self):
-        """colr must have a valid color, strange as though that may sound."""
-        colorspace = -1
-        approx = 0
-        colr = ColourSpecificationBox(colorspace=colorspace,
-                                      approximation=approx)
+        """
+        SCENARIO:  A colr box has an invalid colorspace.
+
+        EXPECTED RESULT:  An InvalidJp2kError is raised when attempting to
+        write the box.
+        """
+        with warnings.catch_warnings():
+            # A warning is issued due to the bad colorspace.
+            warnings.simplefilter("ignore")
+            colr = ColourSpecificationBox(colorspace=-1, approximation=0)
+
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 colr.write(tfile)
 
     def test_write_colr_with_bad_method(self):
         """
         A colr box has an invalid method.
 
-        Expect an IOError when trying to write to file.
+        Expect an InvalidJp2kError when trying to write to file.
         """
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
             colr = ColourSpecificationBox(colorspace=SRGB, method=5)
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 colr.write(tfile)
 
 
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestPaletteBox(unittest.TestCase):
+class TestPaletteBox(fixtures.TestCommon):
     """Test suite for pclr box instantiation."""
 
-    def setUp(self):
-        pass
-
-    def tearDown(self):
-        pass
-
     def test_writing_with_different_bitdepths(self):
         """Bitdepths must be the same when writing."""
         palette = np.array([[255, 0, 255], [0, 255, 0]], dtype=np.uint16)
@@ -548,8 +531,8 @@ class TestPaletteBox(unittest.TestCase):
         signed = (False, False, False)
         pclr = glymur.jp2box.PaletteBox(palette, bits_per_component=bps,
                                         signed=signed)
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 pclr.write(tfile)
 
     def test_signed_components(self):
@@ -579,24 +562,16 @@ class TestPaletteBox(unittest.TestCase):
 
         # Seek back to point after L, T
         b.seek(8)
-        with self.assertRaises(IOError):
+        with self.assertRaises(InvalidJp2kError):
             PaletteBox.parse(b, 8, 20)
 
 
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestAppend(unittest.TestCase):
+class TestAppend(fixtures.TestCommon):
     """Tests for append method."""
 
-    def setUp(self):
-        self.j2kfile = glymur.data.goodstuff()
-        self.jp2file = glymur.data.nemo()
-
-    def tearDown(self):
-        pass
-
     def test_append_xml(self):
         """Should be able to append an XML box."""
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             shutil.copyfile(self.jp2file, tfile.name)
 
             jp2 = Jp2k(tfile.name)
@@ -615,7 +590,7 @@ class TestAppend(unittest.TestCase):
 
     def test_only_jp2_allowed_to_append(self):
         """Only JP2 files are allowed to be appended."""
-        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
+        with open(self.temp_j2k_filename, mode="wb") as tfile:
             shutil.copyfile(self.j2kfile, tfile.name)
 
             j2k = Jp2k(tfile.name)
@@ -624,7 +599,7 @@ class TestAppend(unittest.TestCase):
             # files.
             the_xml = ET.fromstring('<?xml version="1.0"?><data>0</data>')
             xmlbox = glymur.jp2box.XMLBox(xml=the_xml)
-            with self.assertRaises(IOError):
+            with self.assertRaises(RuntimeError):
                 j2k.append(xmlbox)
 
     def test_length_field_is_zero(self):
@@ -634,7 +609,7 @@ class TestAppend(unittest.TestCase):
         handled properly, the appended box is never seen.
         """
         baseline_jp2 = Jp2k(self.jp2file)
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 # Everything up until the jp2c box.
                 offset = baseline_jp2.box[-1].offset
@@ -665,7 +640,7 @@ class TestAppend(unittest.TestCase):
 
     def test_append_allowable_boxes(self):
         """Only XML boxes are allowed to be appended."""
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             shutil.copyfile(self.jp2file, tfile.name)
 
             jp2 = Jp2k(tfile.name)
@@ -674,22 +649,13 @@ class TestAppend(unittest.TestCase):
             uuid_instance = UUID('00000000-0000-0000-0000-000000000000')
             data = b'0123456789'
             uuidbox = glymur.jp2box.UUIDBox(uuid_instance, data)
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 jp2.append(uuidbox)
 
 
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestWrap(unittest.TestCase):
+class TestWrap(fixtures.TestCommon):
     """Tests for wrap method."""
 
-    def setUp(self):
-        self.j2kfile = glymur.data.goodstuff()
-        self.jp2file = glymur.data.nemo()
-        self.jpxfile = glymur.data.jpxfile()
-
-    def tearDown(self):
-        pass
-
     def verify_wrapped_raw(self, jp2file):
         """Shared fixture"""
         jp2 = Jp2k(jp2file)
@@ -743,7 +709,7 @@ class TestWrap(unittest.TestCase):
     def test_wrap(self):
         """basic test for rewrapping a j2c file, no specified boxes"""
         j2k = Jp2k(self.j2kfile)
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name)
             self.verify_wrapped_raw(tfile.name)
 
@@ -753,7 +719,7 @@ class TestWrap(unittest.TestCase):
         # Use only the signature, file type, header, and 1st codestream.
         lst = [0, 1, 2, 5]
         boxes = [jpx.box[idx] for idx in lst]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             jp2 = jpx.wrap(tfile.name, boxes=boxes)
 
         # Verify the outer boxes.
@@ -771,14 +737,14 @@ class TestWrap(unittest.TestCase):
     def test_wrap_jp2(self):
         """basic test for rewrapping a jp2 file, no specified boxes"""
         j2k = Jp2k(self.jp2file)
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             jp2 = j2k.wrap(tfile.name)
         boxes = [box.box_id for box in jp2.box]
         self.assertEqual(boxes, ['jP  ', 'ftyp', 'jp2h', 'jp2c'])
 
     def test_wrap_jp2_Lzero(self):
         """Wrap jp2 with jp2c box length is zero"""
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 tfile.write(ifile.read())
             # Rewrite with codestream length as zero.
@@ -787,14 +753,15 @@ class TestWrap(unittest.TestCase):
             tfile.flush()
             jp2 = Jp2k(tfile.name)
 
-            with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile2:
-                jp2 = jp2.wrap(tfile2.name)
+        file2 = os.path.join(self.test_dir, 'file2.jp2')
+        jp2 = jp2.wrap(file2)
+
         boxes = [box for box in jp2.box]
         self.assertEqual(boxes[3].length, 1132296)
 
     def test_wrap_jp2_Lone(self):
         """Wrap jp2 with jp2c box length is 1, implies Q field"""
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 tfile.write(ifile.read(3223))
                 # Write new L, T, Q fields
@@ -805,8 +772,9 @@ class TestWrap(unittest.TestCase):
             tfile.flush()
             jp2 = Jp2k(tfile.name)
 
-            with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile2:
-                jp2 = jp2.wrap(tfile2.name)
+        file2 = os.path.join(self.test_dir, 'file2.jp2')
+        jp2 = jp2.wrap(file2)
+
         boxes = [box for box in jp2.box]
         self.assertEqual(boxes[3].length, 1132296 + 8)
 
@@ -815,19 +783,19 @@ class TestWrap(unittest.TestCase):
         jp2 = Jp2k(self.jp2file)
         boxes = [box for box in jp2.box]
         boxes[1].compatibility_list = ['jpx ']
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_empty_jp2h(self):
         """JP2H box list cannot be empty."""
         jp2 = Jp2k(self.jp2file)
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             boxes = jp2.box
             # Right here the jp2h superbox has two child boxes.  Empty out that
             # list to trigger the error.
             boxes[2].box = []
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_default_layout_with_boxes(self):
@@ -845,7 +813,7 @@ class TestWrap(unittest.TestCase):
                                        width=width,
                                        num_components=num_components),
                         ColourSpecificationBox(colorspace=glymur.core.SRGB)]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             j2k.wrap(tfile.name, boxes=boxes)
             self.verify_wrapped_raw(tfile.name)
 
@@ -864,8 +832,8 @@ class TestWrap(unittest.TestCase):
                         ImageHeaderBox(height=height,
                                        width=width,
                                        num_components=num_components)]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_first_boxes_jp_and_ftyp(self):
@@ -885,8 +853,8 @@ class TestWrap(unittest.TestCase):
                               num_components=num_components)
         jp2h.box = [ihdr, colr]
         boxes = [ftyp, jp2b, jp2h, jp2c]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_pclr_not_in_jp2h(self):
@@ -912,8 +880,8 @@ class TestWrap(unittest.TestCase):
                               num_components=num_components)
         jp2h.box = [ihdr, colr]
         boxes = [jp2b, ftyp, jp2h, jp2c, pclr]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_jp2h_not_preceeding_jp2c(self):
@@ -933,8 +901,8 @@ class TestWrap(unittest.TestCase):
                               num_components=num_components)
         jp2h.box = [ihdr, colr]
         boxes = [jp2b, ftyp, jp2c, jp2h]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_missing_codestream(self):
@@ -952,22 +920,22 @@ class TestWrap(unittest.TestCase):
                               num_components=num_components)
         jp2h.box = [ihdr]
         boxes = [jp2k, ftyp, jp2h]
-        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 j2k.wrap(tfile.name, boxes=boxes)
 
     def test_wrap_jpx_to_jp2_with_unadorned_jpch(self):
         """A JPX file rewrapped with plain jpch is not allowed."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile1:
+        with open(self.temp_jp2_filename, mode='wb') as tfile1:
             jpx = Jp2k(self.jpxfile)
             boxes = [jpx.box[0], jpx.box[1], jpx.box[2],
                      glymur.jp2box.ContiguousCodestreamBox()]
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 jpx.wrap(tfile1.name, boxes=boxes)
 
     def test_wrap_jpx_to_jp2_with_incorrect_jp2c_offset(self):
         """Reject A JPX file rewrapped with bad jp2c offset."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile1:
+        with open(self.temp_jp2_filename, mode='wb') as tfile1:
             jpx = Jp2k(self.jpxfile)
             jpch = jpx.box[5]
 
@@ -975,12 +943,12 @@ class TestWrap(unittest.TestCase):
             jpch.offset = 901
             jpch.length = 313274
             boxes = [jpx.box[0], jpx.box[1], jpx.box[2], jpch]
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 jpx.wrap(tfile1.name, boxes=boxes)
 
     def test_wrap_jpx_to_jp2_with_correctly_specified_jp2c(self):
         """Accept A JPX file rewrapped with good jp2c."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile1:
+        with open(self.temp_jp2_filename, mode='wb') as tfile1:
             jpx = Jp2k(self.jpxfile)
             jpch = jpx.box[5]
 
@@ -1004,7 +972,7 @@ class TestWrap(unittest.TestCase):
 
     def test_full_blown_jpx(self):
         """Rewrap a jpx file."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile1:
+        with open(self.temp_jp2_filename, mode='wb') as tfile1:
             jpx = Jp2k(self.jpxfile)
             idx = (list(range(5)) + list(range(9, 12)) + list(range(6, 9))
                    + [12])
