@@ -4,6 +4,10 @@ Test suite specifically targeting JPX box layout.
 """
 # Standard library imports ...
 import ctypes
+try:
+    import importlib.resources as ir
+except ImportError:
+    import importlib_resources as ir
 from io import BytesIO
 import os
 import struct
@@ -14,27 +18,24 @@ import warnings
 
 # Third party library imports ...
 try:
-    import lxml.etree as ET
+    from lxml import etree as ET
 except ImportError:
     import xml.etree.ElementTree as ET
-import pkg_resources as pkg
 
 # Local imports ...
 import glymur
 from glymur import Jp2k
 from glymur.jp2box import DataEntryURLBox, FileTypeBox, JPEG2000SignatureBox
 from glymur.jp2box import DataReferenceBox, FragmentListBox, FragmentTableBox
-from glymur.jp2box import ColourSpecificationBox
+from glymur.jp2box import ColourSpecificationBox, InvalidJp2kError
+from . import fixtures, data
 
 
-@unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
-class TestJPXWrap(unittest.TestCase):
+class TestJPXWrap(fixtures.TestCommon):
     """Test suite for wrapping JPX files."""
 
     def setUp(self):
-        self.jpxfile = glymur.data.jpxfile()
-        self.jp2file = glymur.data.nemo()
-        self.j2kfile = glymur.data.goodstuff()
+        super(TestJPXWrap, self).setUp()
 
         raw_xml = b"""<?xml version="1.0"?>
         <data>
@@ -46,9 +47,9 @@ class TestJPXWrap(unittest.TestCase):
                 <neighbor name="Switzerland" direction="W"/>
             </country>
         </data>"""
-        with tempfile.NamedTemporaryFile(suffix=".xml", delete=False) as tfile:
+        self.xmlfile = os.path.join(self.test_dir, 'liechtenstein.xml')
+        with open(self.xmlfile, mode="wb") as tfile:
             tfile.write(raw_xml)
-            tfile.flush()
         self.xmlfile = tfile.name
 
     def tearDown(self):
@@ -56,7 +57,8 @@ class TestJPXWrap(unittest.TestCase):
 
     def test_jpx_ftbl_no_codestream(self):
         """Can have a jpx with no codestream."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile1:
+        file1 = os.path.join(self.test_dir, 'file1.jp2')
+        with open(file1, mode='wb') as tfile1:
             with open(self.jp2file, 'rb') as fptr:
                 tfile1.write(fptr.read())
             tfile1.flush()
@@ -79,7 +81,8 @@ class TestJPXWrap(unittest.TestCase):
             url1 = DataEntryURLBox(0, [0, 0, 0], 'file://' + tfile1.name)
             url1_name_len = len(url1.url) + 1
 
-            with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile2:
+            file2 = os.path.join(self.test_dir, 'file2.jp2')
+            with open(file2, mode='wb') as tfile2:
 
                 j2k = Jp2k(self.j2kfile)
                 jp2_2 = j2k.wrap(tfile2.name)
@@ -97,7 +100,8 @@ class TestJPXWrap(unittest.TestCase):
                                      compatibility_list=['jpx ',
                                                          'jp2 ', 'jpxb']),
                          jp2h]
-                with tempfile.NamedTemporaryFile(suffix='.jpx') as tjpx:
+                jpxfile = os.path.join(self.test_dir, 'file3.jpx')
+                with open(jpxfile, mode='wb') as tjpx:
                     for box in boxes:
                         box.write(tjpx)
 
@@ -127,8 +131,8 @@ class TestJPXWrap(unittest.TestCase):
 
         boxes.append(glymur.jp2box.AssociationBox())
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_jpch_jplh(self):
@@ -145,7 +149,7 @@ class TestJPXWrap(unittest.TestCase):
         jplh = glymur.jp2box.CompositingLayerHeaderBox()
         boxes.append(jplh)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             jpx = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jpx.box[-2].box_id, 'jpch')
@@ -167,7 +171,7 @@ class TestJPXWrap(unittest.TestCase):
         cgrp = glymur.jp2box.ColourGroupBox(box=box)
         boxes.append(cgrp)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             jpx = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jpx.box[-1].box_id, 'cgrp')
@@ -189,8 +193,8 @@ class TestJPXWrap(unittest.TestCase):
         cgrp = glymur.jp2box.ColourGroupBox(box=box)
         boxes.append(cgrp)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_cgrp_neg(self):
@@ -209,8 +213,8 @@ class TestJPXWrap(unittest.TestCase):
         cgrp = glymur.jp2box.ColourGroupBox(box=box)
         boxes.append(cgrp)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_ftbl(self):
@@ -234,7 +238,7 @@ class TestJPXWrap(unittest.TestCase):
         ftbl = glymur.jp2box.FragmentTableBox(box=[flst])
         boxes.append(ftbl)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             jpx = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jpx.box[1].compatibility_list, ['jp2 ', 'jpxb'])
@@ -258,7 +262,7 @@ class TestJPXWrap(unittest.TestCase):
         asoc = glymur.jp2box.AssociationBox([nlst, xmlb])
         boxes.append(asoc)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             jpx = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jpx.box[1].compatibility_list, ['jp2 ', 'jpxb'])
@@ -288,7 +292,7 @@ class TestJPXWrap(unittest.TestCase):
         asoc = glymur.jp2box.AssociationBox([nlst, xmlb, lblb])
         boxes.append(asoc)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             jpx = jp2.wrap(tfile.name, boxes=boxes)
 
             self.assertEqual(jpx.box[1].compatibility_list, ['jp2 ', 'jpx '])
@@ -311,8 +315,8 @@ class TestJPXWrap(unittest.TestCase):
         dref = glymur.jp2box.DataReferenceBox()
         boxes.append(dref)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_deurl_child_of_dtbl(self):
@@ -331,8 +335,8 @@ class TestJPXWrap(unittest.TestCase):
 
         boxes.append(dref)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_only_one_data_reference(self):
@@ -351,8 +355,8 @@ class TestJPXWrap(unittest.TestCase):
         boxes.append(dref)
         boxes.append(dref)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_lbl_at_top_level(self):
@@ -368,8 +372,8 @@ class TestJPXWrap(unittest.TestCase):
         # Put it inside the jp2 header box.
         boxes[2].box.append(lblb)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_data_reference_in_subbox(self):
@@ -389,8 +393,8 @@ class TestJPXWrap(unittest.TestCase):
         # Put it inside the jp2 header box.
         boxes[2].box.append(dref)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
     def test_jp2_to_jpx_sans_jp2_compatibility(self):
@@ -409,7 +413,7 @@ class TestJPXWrap(unittest.TestCase):
         asoc = glymur.jp2box.AssociationBox([nlst, xmlb])
         boxes.append(asoc)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             with self.assertRaises(RuntimeError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
@@ -425,59 +429,52 @@ class TestJPXWrap(unittest.TestCase):
         asoc = glymur.jp2box.AssociationBox([nlst, xmlb])
         boxes.append(asoc)
 
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             with self.assertRaises(RuntimeError):
                 jp2.wrap(tfile.name, boxes=boxes)
 
 
-@unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
-class TestJPX(unittest.TestCase):
+class TestJPX(fixtures.TestCommon):
     """Test suite for other JPX boxes."""
 
-    def setUp(self):
-        self.jp2file = glymur.data.nemo()
-        self.jpxfile = glymur.data.jpxfile()
-
-    def tearDown(self):
-        pass
-
     def test_reader_requirements_box(self):
         """
         Simple test for parsing a reader requirements box.
         """
-        file = os.path.join('data', 'text_GBR.jp2')
-        file = pkg.resource_filename(__name__, file)
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            j = Jp2k(file)
-            self.assertEqual(len(j.box[2].vendor_feature), 4)
+        with ir.path(data, 'text_GBR.jp2') as path:
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore')
+                j = Jp2k(path)
+                self.assertEqual(len(j.box[2].vendor_feature), 4)
 
     @unittest.skipIf(sys.hexversion < 0x03000000, "assertWarns is PY3K")
     def test_reader_requirements_box_writing(self):
         """
         If a box does not have writing specifically enabled, must error out.
         """
-        file = os.path.join('data', 'text_GBR.jp2')
-        file = pkg.resource_filename(__name__, file)
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            j = Jp2k(file)
-            box = j.box[2]
+        with ir.path(data, 'text_GBR.jp2') as path:
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore')
+                j = Jp2k(path)
 
-            b = BytesIO()
-            with self.assertRaises(NotImplementedError):
-                box.write(b)
+        box = j.box[2]
+        b = BytesIO()
+
+        with self.assertRaises(NotImplementedError):
+            box.write(b)
 
     def test_flst_lens_not_the_same(self):
         """A fragment list box items must be the same length."""
         offset = [89]
         length = [1132288]
         reference = [0, 0]
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
+
+        with warnings.catch_warnings(record=True) as w:
             flst = glymur.jp2box.FragmentListBox(offset, length, reference)
+            assert issubclass(w[-1].category, UserWarning)
+
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 flst.write(tfile)
 
     def test_flst_offsets_not_positive(self):
@@ -489,7 +486,7 @@ class TestJPX(unittest.TestCase):
             warnings.simplefilter('ignore')
             flst = glymur.jp2box.FragmentListBox(offset, length, reference)
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises((IOError, OSError)):
+            with self.assertRaises(InvalidJp2kError):
                 flst.write(tfile)
 
     def test_flst_lengths_not_positive(self):
@@ -501,14 +498,14 @@ class TestJPX(unittest.TestCase):
             warnings.simplefilter('ignore')
             flst = glymur.jp2box.FragmentListBox(offset, length, reference)
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 flst.write(tfile)
 
     def test_ftbl_boxes_empty(self):
         """A fragment table box must have at least one child box."""
         ftbl = glymur.jp2box.FragmentTableBox()
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 ftbl.write(tfile)
 
     def test_ftbl_child_not_flst(self):
@@ -516,7 +513,7 @@ class TestJPX(unittest.TestCase):
         free = glymur.jp2box.FreeBox()
         ftbl = glymur.jp2box.FragmentTableBox(box=[free])
         with tempfile.TemporaryFile() as tfile:
-            with self.assertRaises(IOError):
+            with self.assertRaises(InvalidJp2kError):
                 ftbl.write(tfile)
 
     def test_data_reference_requires_dtbl(self):
@@ -534,8 +531,8 @@ class TestJPX(unittest.TestCase):
         dref = glymur.jp2box.DataReferenceBox([deurl1, deurl2])
         boxes.append(dref)
 
-        with tempfile.NamedTemporaryFile(suffix='.jpx') as tfile:
-            with self.assertRaises(IOError):
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
+            with self.assertRaises(InvalidJp2kError):
                 jpx1.wrap(tfile.name, boxes=boxes)
 
     def test_dtbl_free(self):
@@ -545,7 +542,7 @@ class TestJPX(unittest.TestCase):
         version = (0, 0, 0)
         url1 = 'file:////usr/local/bin'
         url2 = 'http://glymur.readthedocs.org' + chr(0) * 3
-        with tempfile.NamedTemporaryFile(suffix='.jpx') as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             with open(self.jpxfile, 'rb') as ifile:
                 tfile.write(ifile.read())
 
@@ -572,7 +569,7 @@ class TestJPX(unittest.TestCase):
     def test_ftbl(self):
         """Verify that we can interpret Fragment Table boxes."""
         # Copy the existing JPX file, add a fragment table box onto the end.
-        with tempfile.NamedTemporaryFile(suffix='.jpx') as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             with open(self.jpxfile, 'rb') as ifile:
                 tfile.write(ifile.read())
             write_buffer = struct.pack('>I4s', 32, b'ftbl')
@@ -629,7 +626,7 @@ class TestJPX(unittest.TestCase):
 
         # Ok, done with the box, we can now insert it into the jpx file after
         # the ftyp box.
-        with tempfile.NamedTemporaryFile(suffix=".jpx") as ofile:
+        with open(self.temp_jpx_filename, mode="wb") as ofile:
             with open(self.jpxfile, 'rb') as ifile:
                 ofile.write(ifile.read(40))
                 ofile.write(rreq_buffer)
