@@ -3,13 +3,15 @@
 Test suite for printing.
 """
 # Standard library imports ...
+try:
+    import importlib.resources as ir
+except ImportError:
+    import importlib_resources as ir
 from io import BytesIO
 import os
-import pkg_resources as pkg
 import re
 import struct
 import sys
-import tempfile
 import unittest
 from uuid import UUID
 import warnings
@@ -26,48 +28,44 @@ else:
 
 # Third party imports ...
 import numpy as np
-
 try:
-    import lxml.etree as ET
+    from lxml import etree as ET
 except ImportError:
     import xml.etree.ElementTree as ET
 
+# Local imports
 import glymur
 from glymur.codestream import LRCP, WAVELET_XFORM_5X3_REVERSIBLE
 from glymur.core import COLOR, RED, GREEN, BLUE, RESTRICTED_ICC_PROFILE
 from glymur.jp2box import BitsPerComponentBox, ColourSpecificationBox
 from glymur.jp2box import LabelBox
 from glymur import Jp2k, command_line
-from . import fixtures
-from .fixtures import (WINDOWS_TMP_FILE_MSG,
-                       OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG)
+from . import fixtures, data
+from .fixtures import OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG
 
 
-@unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
-class TestPrinting(unittest.TestCase):
+class TestPrinting(fixtures.TestCommon):
     """
     Tests for verifying how printing works.
     """
     def setUp(self):
-        self.jpxfile = glymur.data.jpxfile()
-        self.jp2file = glymur.data.nemo()
-        self.j2kfile = glymur.data.goodstuff()
+        super(TestPrinting, self).setUp()
 
         # Reset printoptions for every test.
         glymur.reset_option('all')
 
     def tearDown(self):
+        super(TestPrinting, self).tearDown()
         glymur.reset_option('all')
 
     def test_bad_color_specification(self):
         """
         Invalid channel type should not prevent printing.
         """
-        relfile = os.path.join('data', 'issue392.jp2')
-        file = pkg.resource_filename(__name__, relfile)
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            jp2 = Jp2k(file)
+        with ir.path(data, 'issue392.jp2') as path:
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore')
+                jp2 = Jp2k(path)
         str(jp2)
 
     def test_palette(self):
@@ -285,12 +283,15 @@ class TestPrinting(unittest.TestCase):
 
     def test_invalid_colorspace(self):
         """
-        An invalid colorspace shouldn't cause an error.
+        SCENARIO:  An invalid colorspace shouldn't cause an error when
+        printing.
 
-        Original test file was edf_c2_1103421.jp2
+        EXPECTED RESULT:  No error, although there is a warning.
         """
-        colr = ColourSpecificationBox(colorspace=276)
-        colr.colorspace = 276
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            colr = ColourSpecificationBox(colorspace=276)
+
         str(colr)
 
     def test_label_box_short(self):
@@ -361,7 +362,7 @@ class TestPrinting(unittest.TestCase):
 
     def test_unknown_superbox(self):
         """Verify that we can handle an unknown superbox."""
-        with tempfile.NamedTemporaryFile(suffix='.jpx') as tfile:
+        with open(self.temp_jpx_filename, mode='wb') as tfile:
             with open(self.jpxfile, 'rb') as ifile:
                 tfile.write(ifile.read())
 
@@ -401,9 +402,13 @@ class TestPrinting(unittest.TestCase):
         # Construct a fake file with an asoc and a label box, as
         # OpenJPEG doesn't have such a file.
         data = glymur.Jp2k(self.jp2file)[::2, ::2]
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
-            with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile2:
-                glymur.Jp2k(tfile.name, data=data)
+
+        # Create JP2 file with just the basic boxes.
+        vanilla_jp2_filename = os.path.join(self.test_dir, 'vanilla.jp2')
+        glymur.Jp2k(vanilla_jp2_filename, data=data)
+
+        with open(vanilla_jp2_filename, mode='rb') as tfile:
+            with open(self.temp_jp2_filename, mode='wb') as tfile2:
 
                 # Offset of the codestream is where we start.
                 wbuffer = tfile.read(77)
@@ -787,10 +792,9 @@ class TestPrinting(unittest.TestCase):
 
         self.assertEqual(actual, expected)
 
-    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
     def test_less_common_boxes(self):
         """verify uinf, ulst, url, res, resd, resc box printing"""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 # Everything up until the jp2c box.
                 wbuffer = ifile.read(77)
@@ -990,7 +994,7 @@ class TestPrinting(unittest.TestCase):
                      "Ordered dicts not printing well in 2.7")
     def test_exif_uuid(self):
         """Verify printing of exif information"""
-        with tempfile.NamedTemporaryFile(suffix='.jp2', mode='wb') as tfile:
+        with open(self.temp_jp2_filename, mode='wb') as tfile:
 
             with open(self.jp2file, 'rb') as ifptr:
                 tfile.write(ifptr.read())
@@ -1319,7 +1323,7 @@ class TestPrinting(unittest.TestCase):
         verify that we print precincts correctly
         """
         data = Jp2k(self.jp2file)[:]
-        with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
+        with open(self.temp_j2k_filename, mode='wb') as tfile:
             j = Jp2k(tfile.name, data=data, psizes=[(128, 128)] * 3)
 
             # Should be three layers.
@@ -1575,9 +1579,8 @@ class TestJp2dump(unittest.TestCase):
         """
         Warnings about invalid JP2/J2K syntax should be suppressed until end
         """
-        file = os.path.join('data', 'edf_c2_1178956.jp2')
-        file = pkg.resource_filename(__name__, file)
-        actual = self.run_jp2dump(['', '-x', file])
+        with ir.path(data, 'edf_c2_1178956.jp2') as path:
+            actual = self.run_jp2dump(['', '-x', str(path)])
 
         # The "CME marker segment" part is the last segment in the codestream
         # header.
